#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <modbus.h>

#define MB_SERVER_ID 17
#define MB_TEMPERATURE_REGISTER_ADDRESS 0x08

int main(int argc,char** argv)
{
	modbus_t *ctx;
	modbus_mapping_t *mb_mapping;
	uint8_t *query;
	int rc, temperature, temperature_flag;

	if (argc < 2) {
		printf("usage: %s SerialPortDeviceFile\n", argv[0]);
		return -1;
	}

	ctx = modbus_new_rtu(argv[1], 115200, 'N', 8, 1);
	if (ctx == NULL) {
		fprintf(stderr, "modbus_new_rtu: %s\n", modbus_strerror(errno));
		return -1;
	}

	modbus_set_debug(ctx, TRUE);
	modbus_set_slave(ctx, MB_SERVER_ID);

	mb_mapping = modbus_mapping_new(0, 0, 0,
		      			MB_TEMPERATURE_REGISTER_ADDRESS + 1);
	if (mb_mapping == NULL) {
		fprintf(stderr, "modbus_mapping_new: %s\n", modbus_strerror(errno));
		modbus_free(ctx);
		return -1;
	}

	rc = modbus_connect(ctx);
	if (rc == -1) {
		fprintf(stderr, "modbus_connect: %s\n", modbus_strerror(errno));
		modbus_free(ctx);
		return -1;
	}

	temperature_flag = TRUE;
	temperature = 0;
	query = malloc(MODBUS_RTU_MAX_ADU_LENGTH);
	for (;;) {
		mb_mapping->tab_input_registers[MB_TEMPERATURE_REGISTER_ADDRESS] = temperature;

	rc = modbus_receive(ctx, query);
	if (rc == -1)
		break;

        rc = modbus_reply(ctx, query, rc, mb_mapping);
	if (rc == -1)
		break;
 

	printf("temperature = %d\n", temperature);
	if (temperature_flag == TRUE) {
		if (++temperature >= 30)
			temperature_flag = FALSE;
	} else {
		if (--temperature <= 0)
			temperature_flag = TRUE;
		}
	}

	printf("Quit the loop: %s\n", modbus_strerror(errno));

	modbus_mapping_free(mb_mapping);
	free(query);
	modbus_free(ctx);
}
